option show_stats;
option solver gurobi_ampl;

## Model parameters
param p; # Number of processors
set P = 1..p; # Set of processors
param M; # Crown maximal makespan
param n; # Number of tasks
set T = 1..n; # Task set
param Wi{i in T}; # Task width
param Tau{i in T}; # Task work
param e{i in T, j in P}; # Task efficiency distribution
set F;
param alpha; # Frequency cost parameter
param freq{j in P};
param w{i in T, j in P};

param wi {i in T} = 1;

## Task-to_core mapping
## makespan for each core before frequency scaling
param unscaled_makespan_per_core {j in P} = sum{i in T} w[i,j] * Tau[i];
param quality = max{j in P} unscaled_makespan_per_core[j];
param schedule {j in P, i in T} = w[i,j] * i;

var x{i in T, k in F} binary;
	subject to one_freq_per_task{i in T}: sum {k in F} x[i,k] = 1;
	subject to stuff{j in P}: sum{i in T} sum{k in F} w[i,j] * x[i,k] * Tau[i] / k <= M;

## discrete frequency
var frequency{i in T} = sum{k in F} x[i,k] * k;

var energy = sum{i in T} sum{k in F} (x[i, k] * Tau[i]) * frequency[i]^alpha;
var pseudo_energy = sum{i in T} Tau[i] * frequency[i]^2;
minimize target: sum{j in P} pseudo_energy;

## Comment the three lines above and uncomment these to make the rescaling phase neutral and output continuous frequencies
#param frequency{i in T} = sum{j in P} w[i,j] * freq[j];
#param energy = sum{i in T} (Tau[i] / frequency[i]) * frequency[i]^alpha;
#var dummy_var;
#	subject to dummy_constraint: dummy_var >= 0;
#minimize dummy: dummy_var;


