option show_stats;
option solver gurobi_ampl;

## Model parameters
param p; # Number of processors
set P = 1..p; # Set of processors
param M; # Crown maximal makespan
param n; # Number of tasks
set T = 1..n; # Task set
param Wi{i in T}; # Task width
param Tau{i in T}; # Task work
param e{i in T, j in P}; # Task efficiency distribution

set F;
param alpha; # Frequency cost parameter

param Fin;
set Fi{i in 1..Fin};
param Funit;
param c{t1 in T, t2 in T};

## Task-to_core mapping
var _w{i in T, j in P} binary;
var w{i in T,j in P} = round(_w[i,j]); # make sure output is integer digits
## Tasks are mapped to exactly one core
subject to one_core_per_task {i in T}: sum{j in P} _w[i,j] = 1;

## makespan for each core before frequency scaling
var unscaled_makespan_per_core {j in P} = sum{i in T} _w[i,j] * Tau[i];
var makespan;
subject to slack {j in P}: makespan >= unscaled_makespan_per_core[j];

## Ideal frequency
var freq{j in P} = unscaled_makespan_per_core[j] / M;
var energy = sum{ j in P} M * freq[j]^alpha;

var pseudo_energy = sum {j in P} M * freq[j]^2;

minimize target: pseudo_energy;
#minimize target: makespan;

