option show_stats;
option solver gurobi_ampl;

## Model parameters
param p; # Number of processors
set P = 1..p; # Set of processors
param M; # Crown maximal makespan
param n; # Number of tasks
set T = 1..n; # Task set
param Wi{i in T}; # Task width
param Tau{i in T}; # Task work
param e{i in T, j in P}; # Task efficiency distribution
set F;
param alpha; # Frequency cost parameter

param Fin;
set Fi{i in 1..Fin};
param Funit;
param c{t1 in T, t2 in T};

## There are at most as many levels as tasks
set L = 1..n;

## Xu et al have a scaling factor theta for frequency. We assume here theta(f) = f;
param theta{i in T, k in F} = k;
param PC{k in F} = k^alpha;

## Task-to-level-degree-and-frequency mapping 4D matrix
var x {i in T, j in P, l in L, k in F} binary;
	subject to exactly_one_level_and_frequency_per_task{i in T}: sum{j in P} sum{l in L} sum{k in F} x[i,j,l,k] = 1;
	subject to no_more_core_in_level_than_arch{l in L}: sum{i in T} sum{j in P} sum{k in F} x[i,j,l,k] * j <= p;
	subject to level_height_is_init_task_height{i in T, l in L}: sum{j in P} sum{k in F} x[i,j,l,k] * Tau[i] / (j * e[i,j] * theta[i, k]) <= sum{j in P} sum{k in F} x[l,j,l,k] * Tau[l] / (j * e[l,j] * theta[l, k]);
	subject to sum_level_height_lower_than_deadline: sum{i in T} sum{j in P} sum{k in F} x[i,j,i,k] * Tau[i] / (j * e[i,j] * theta[i, k]) <= M;

var energy = sum{i in T} sum{j in P} sum{l in L} sum{k in F} (x[i,j,l,k] * Tau[i] / (j * e[i,j] * k)) * j * PC[k];

var frequency{i in T} = sum{j in P} sum{l in L} sum{k in F} x[i,j,l,k] * k;
var level{i in T} = sum{j in P} sum{l in L} sum{k in F} x[i,j,l,k] * l;
var time{i in L} = sum{j in P} sum{l in L} sum{k in F} x[i,j,l,k] * Tau[l] / (j * e[l,j] * theta[l, k]);
var tasks{l in L, i in T} = sum{j in P} sum{k in F} x[i,j,l,k] * i;
var wi{i in T} = sum{j in P} sum{l in L} sum{k in F} x[i,j,l,k] * j;
var quality = sum{i in T} sum{j in P} sum{k in F} x[i,j,i,k] * Tau[i] / (j * e[i,j]);

minimize target: energy;

